"""A lightweight YouTube data scraper."""

__description__ = "A lightweight YouTube data scraper."
__url__ = "https://github.com/anekobtw/youthon"

__title__ = "youthon"
__author__ = "anekobtw"
__license__ = "MIT"
__version__ = "0.4.2"

from youthon.channel import Channel
from youthon.playlist import Playlist
from youthon.video import Video

__all__ = ("Channel", "Video", "Playlist")



import requests

from youthon.funcs import get_initial_player_response


class Video:
    def __init__(self, url: str) -> None:
        headers = {"User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/96.0.4664.45 Safari/537.36", "X-Amzn-Trace-Id": "Root=1-61acac03-6279b8a6274777eb44d81aae", "X-Client-Data": "CJW2yQEIpLbJAQjEtskBCKmdygEIuevKAQjr8ssBCOaEzAEItoXMAQjLicwBCKyOzAEI3I7MARiOnssB"}
        response = requests.get(url, headers=headers)

        video_details = get_initial_player_response(response)["videoDetails"]

        self.title: str = video_details["title"]
        self.video_url: str = f'https://www.youtube.com/watch?v={video_details["videoId"]}'
        self.thumbnail_url: str = video_details["thumbnail"]["thumbnails"][-1]["url"]
        self.description: str = video_details["shortDescription"]
        self.author_name: str = video_details["author"]

        self.views: int = int(video_details.get("viewCount", 0))
        self.length_seconds: int = int(video_details.get("lengthSeconds", 0))

        self.allowRatings: bool = video_details["allowRatings"]
        self.isLiveContent: bool = video_details["isLiveContent"]
        self.is_private: bool = video_details["isPrivate"]

        self.keywords: list[str] = video_details.get("keywords", [])



import requests
from bs4 import BeautifulSoup

from youthon.funcs import get_yt_initial_data


class Playlist:
    def __init__(self, url: str) -> None:
        response = requests.get(url)
        soup = BeautifulSoup(response.content, "html.parser")
        initial_data = get_yt_initial_data(response)

        playlist_data = initial_data["contents"]["twoColumnWatchNextResults"]["playlist"]["playlist"]

        self.title: str = playlist_data.get("title", "Unknown Title")
        self.author_url: str = soup.find_all("span")[0].find("link")["href"]
        self.playlist_id: str = playlist_data.get("playlistId", "")
        self.playlist_url: str = f"https://www.youtube.com/playlist?list={self.playlist_id}"
        self.total_videos: int = int(playlist_data.get("totalVideos", 0))
        self.is_infinite: bool = playlist_data.get("isInfinite", False)

        self.video_urls = [self.get_video_url(i, playlist_data) for i in range(self.total_videos)]

    def get_video_url(self, video_id: int, playlist_data) -> str:
        try:
            return f'https://youtube.com/watch/?v={playlist_data["contents"][video_id]["playlistPanelVideoRenderer"]["navigationEndpoint"]["watchEndpoint"]["videoId"]}'
        except (IndexError, KeyError):
            return f"Error: Video ID {video_id} not found"



import requests
from bs4 import BeautifulSoup

from youthon.funcs import get_yt_initial_data


class Channel:
    def __init__(self, url: str) -> None:
        headers = {"User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/96.0.4664.45 Safari/537.36", "X-Amzn-Trace-Id": "Root=1-61acac03-6279b8a6274777eb44d81aae", "X-Client-Data": "CJW2yQEIpLbJAQjEtskBCKmdygEIuevKAQjr8ssBCOaEzAEItoXMAQjLicwBCKyOzAEI3I7MARiOnssB"}
        response = requests.get(url, headers=headers)
        soup = BeautifulSoup(response.content, "html.parser")

        meta_tags = soup.find_all("meta")
        initial_data = get_yt_initial_data(response=response)

        self.name = meta_tags[5]["content"]
        self.description = meta_tags[11]["content"]
        self.channel_url = meta_tags[7]["content"]
        self.profile_photo_url = meta_tags[8]["content"]

        metadata_parts = initial_data["header"]["pageHeaderRenderer"]["content"]["pageHeaderViewModel"]["metadata"]["contentMetadataViewModel"]["metadataRows"][1]["metadataParts"]
        self.subscribers_count = metadata_parts[0]["text"]["content"].split(" ")[0]
        self.video_count = metadata_parts[1]["text"]["content"].split(" ")[0]
        self.channel_id = initial_data["metadata"]["channelMetadataRenderer"]["externalId"]

        self.videos_page = f"{self.channel_url}/videos"
        self.shorts_page = f"{self.channel_url}/shorts"
        self.playlists_page = f"{self.channel_url}/playlists"
        self.community_page = f"{self.channel_url}/community"
        self.featured_channels_page = f"{self.channel_url}/channels"
        self.about_page = f"{self.channel_url}/about"



import json
import re

from graphviz import Digraph
from requests import Response


def get_yt_initial_data(response: Response):
    pattern = re.compile(r"ytInitialData\s*=\s*({.*?});", re.DOTALL)
    match = pattern.search(response.text)
    if not match:
        raise Exception("ytInitialData not found in the page")

    json_str = match.group(1)
    return json.loads(json_str)


def get_initial_player_response(response: Response):
    pattern = re.compile(r"var ytInitialPlayerResponse\s*=\s*({.*?});", re.DOTALL)
    match = pattern.search(response.text)
    if not match:
        raise Exception("ytInitialPlayerResponse not found in the page")

    json_str = match.group(1)
    return json.loads(json_str)


def visualize_dict(d, parent_key=None, graph=None):
    if graph is None:
        graph = Digraph()

    for key, value in d.items():
        node_key = f"{parent_key}.{key}" if parent_key else str(key)
        graph.node(node_key, str(key))

        if parent_key:
            graph.edge(parent_key, node_key)

        if isinstance(value, dict):
            visualize_dict(value, node_key, graph)
        else:
            value_node = f"{node_key}.value"
            graph.node(value_node, str(value))
            graph.edge(node_key, value_node)

    return graph
